@@ -7,6 +7,8 @@ import itertools
 import traceback
 import subprocess
 from contextlib import contextmanager, ExitStack
+from enum import Enum
+import ujson
 import pytest
 
 # XXX Using pytest private internals here
@@ -22,50 +24,21 @@ from _pytest.capture import MultiCapture, FDCapture
 from buildstream._frontend import cli as bst_cli
 from buildstream import _yaml
 
-# Special private exception accessor, for test case purposes
-from buildstream._exceptions import BstError, get_last_exception, get_last_task_error
-
 
 # Wrapper for the click.testing result
 class Result():
 
     def __init__(self,
                  exit_code=None,
-                 exception=None,
-                 exc_info=None,
                  output=None,
                  stderr=None):
         self.exit_code = exit_code
-        self.exc = exception
-        self.exc_info = exc_info
         self.output = output
         self.stderr = stderr
-        self.unhandled_exception = False
-
-        # The last exception/error state is stored at exception
-        # creation time in BstError(), but this breaks down with
-        # recoverable errors where code blocks ignore some errors
-        # and fallback to alternative branches.
-        #
-        # For this reason, we just ignore the exception and errors
-        # in the case that the exit code reported is 0 (success).
-        #
-        if self.exit_code != 0:
-
-            # Check if buildstream failed to handle an
-            # exception, topevel CLI exit should always
-            # be a SystemExit exception.
-            #
-            if not isinstance(exception, SystemExit):
-                self.unhandled_exception = True
-
-            self.exception = get_last_exception()
-            self.task_error_domain, \
-                self.task_error_reason = get_last_task_error()
-        else:
-            self.exception = None
-            self.task_error_domain = None
-            self.task_error_reason = None
+        self.main_error_domain = None
+        self.main_error_reason = None
+        self.task_error_domain = None
+        self.task_error_reason = None
 
     # assert_success()
     #
@@ -79,9 +52,6 @@ class Result():
     #
     def assert_success(self, fail_message=''):
         assert self.exit_code == 0, fail_message
-        assert self.exc is None, fail_message
-        assert self.exception is None, fail_message
-        assert self.unhandled_exception is False
 
     # assert_main_error()
     #
@@ -106,23 +76,20 @@ class Result():
             print(
                 """
                 Exit code: {}
-                Exception: {}
                 Domain:    {}
                 Reason:    {}
                 """.format(
                     self.exit_code,
-                    self.exception,
-                    self.exception.domain,
-                    self.exception.reason
+                    self.main_error_domain,
+                    self.main_error_reason
                 ))
-        assert self.exit_code == -1, fail_message
-        assert self.exc is not None, fail_message
-        assert self.exception is not None, fail_message
-        assert isinstance(self.exception, BstError), fail_message
-        assert self.unhandled_exception is False
+        assert self.exit_code != 0, fail_message
 
-        assert self.exception.domain == error_domain, fail_message
-        assert self.exception.reason == error_reason, fail_message
+        test_domain = error_domain.value if isinstance (error_domain, Enum) else error_domain
+        test_reason = error_reason.value if isinstance (error_reason, Enum) else error_reason
+
+        assert self.main_error_domain == test_domain, fail_message
+        assert self.main_error_reason == test_reason, fail_message
 
     # assert_task_error()
     #
@@ -143,14 +110,12 @@ class Result():
                           error_reason,
                           fail_message=''):
 
-        assert self.exit_code == -1, fail_message
-        assert self.exc is not None, fail_message
-        assert self.exception is not None, fail_message
-        assert isinstance(self.exception, BstError), fail_message
-        assert self.unhandled_exception is False
+        test_domain = error_domain.value if isinstance (error_domain, Enum) else error_domain
+        test_reason = error_reason.value if isinstance (error_reason, Enum) else error_reason
 
-        assert self.task_error_domain == error_domain, fail_message
-        assert self.task_error_reason == error_reason, fail_message
+        assert self.exit_code != 0, fail_message
+        assert self.task_error_domain == test_domain, fail_message
+        assert self.task_error_reason == test_reason, fail_message
 
     # get_tracked_elements()
     #
@@ -241,9 +206,21 @@ class Cli():
         if options is None:
             options = []
 
+        if env is None:
+            env = os.environ.copy()
+        else:
+            orig_env = os.environ.copy()
+            orig_env.update (env)
+            env = orig_env
+
         options = self.default_options + options
 
         with ExitStack() as stack:
+
+            # Prepare a tempfile for buildstream to record machine readable error codes
+            error_codes = stack.enter_context (tempfile.NamedTemporaryFile())
+            env['BST_TEST_ERROR_CODES'] = error_codes.name
+
             bst_args = ['--no-colors']
 
             if silent:
@@ -263,21 +240,34 @@ class Cli():
 
             bst_args += args
 
-            if cwd is not None:
-                stack.enter_context(chdir(cwd))
+            cmd = ["bst"] + bst_args
+            process = subprocess.Popen(
+                cmd,
+                env=env,
+                cwd=cwd,
+                stdin=subprocess.DEVNULL,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE,
+            )
+            out, err = process.communicate()
+            result = Result(
+                exit_code=process.poll(),
+                output=out.decode('utf-8'),
+                stderr=err.decode('utf-8')
+            )
 
-            if env is not None:
-                stack.enter_context(environment(env))
-
-            # Ensure we have a working stdout - required to work
-            # around a bug that appears to cause AIX to close
-            # sys.__stdout__ after setup.py
-            try:
-                sys.__stdout__.fileno()
-            except ValueError:
-                sys.__stdout__ = open('/dev/stdout', 'w')
-
-            result = self.invoke(bst_cli, bst_args)
+            #
+            # Collect machine readable error codes from the tmpfile
+            #
+            result_error_codes_string = error_codes.read()
+            result_error_codes = {}
+            if result_error_codes_string:
+                result_error_codes = ujson.loads (result_error_codes_string)
+            if result_error_codes:
+                result.main_error_domain = result_error_codes['main_error_domain']
+                result.main_error_reason = result_error_codes['main_error_reason']
+                result.task_error_domain = result_error_codes['task_error_domain']
+                result.task_error_reason = result_error_codes['task_error_reason']
 
         # Some informative stdout we can observe when anything fails
         if self.verbose:
@@ -289,54 +279,8 @@ class Cli():
             if result.stderr:
                 print("Program stderr was:\n{}".format(result.stderr))
 
-            if result.exc_info and result.exc_info[0] != SystemExit:
-                traceback.print_exception(*result.exc_info)
-
         return result
 
-    def invoke(self, cli, args=None, color=False, **extra):
-        exc_info = None
-        exception = None
-        exit_code = 0
-
-        # Temporarily redirect sys.stdin to /dev/null to ensure that
-        # Popen doesn't attempt to read pytest's dummy stdin.
-        old_stdin = sys.stdin
-        with open(os.devnull) as devnull:
-            sys.stdin = devnull
-            capture = MultiCapture(out=FDCapture(1), err=FDCapture(2), in_=None)
-            capture.start_capturing()
-
-            try:
-                cli.main(args=args or (), prog_name=cli.name, **extra)
-            except SystemExit as e:
-                if e.code != 0:
-                    exception = e
-
-                exc_info = sys.exc_info()
-
-                exit_code = e.code
-                if not isinstance(exit_code, int):
-                    sys.stdout.write('Program exit code was not an integer: ')
-                    sys.stdout.write(str(exit_code))
-                    sys.stdout.write('\n')
-                    exit_code = 1
-            except Exception as e:
-                exception = e
-                exit_code = -1
-                exc_info = sys.exc_info()
-            finally:
-                sys.stdout.flush()
-
-        sys.stdin = old_stdin
-        out, err = capture.readouterr()
-        capture.stop_capturing()
-
-        return Result(exit_code=exit_code,
-                      exception=exception,
-                      exc_info=exc_info,
-                      output=out,
-                      stderr=err)
 
     # Fetch an element state by name by
     # invoking bst show on the project with the CLI
